/*
 * A page mapper providing memory pages to satisfy file accesses.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/access_map.h>
#include <fsserver/accessor.h>
#include <fsserver/page_owner.h>

#include <mutex>



/* A file mapper, associating flexpages with file regions. */

class PageMapper : public PageOwner
{
protected:
    AccessMap _map;
    Accessor *_accessor;
    Pages *_pages;
    unsigned int _attached;

    /* Serialisation of accesses. */

    std::mutex _lock;

    /* Internal flexpage retrieval methods. */

    Flexpage *find(offset_t offset);

    Flexpage *flexpage(offset_t offset);

public:
    explicit PageMapper(Accessor *accessor, Pages *pages);

    /* Accounting methods. */

    void attach();

    unsigned int detach();

    Accessor *accessor()
    { return _accessor; }

    /* Interface for the pager. */

    Flexpage *get(offset_t offset, map_flags_t flags);

    void queue(Flexpage *flexpage);

    void flush_all(offset_t start, offset_t size);

    offset_t get_data_size();

    void set_data_size(offset_t size);

    /* Data transfer methods, implementing PageOwner. */

    void fill(Flexpage *flexpage);

    void flush(Flexpage *flexpage, bool purge);

    /* Interface for the page collection, implementing PageOwner. */

    void remove(Flexpage *flexpage);
};

// vim: tabstop=4 expandtab shiftwidth=4
